#define WIN32_LEAN_AND_MEAN
#include <windows.h>

//function prototypes
int WINAPI WinMain(HINSTANCE, HINSTANCE, LPSTR, int);
bool CreateMainWindow(HINSTANCE, int);
LRESULT WINMAIN WinProc(HWND, UINT, WPARAM, LPARAM);

//Global variable
HINSTANCE hinst;

//Constants
const char CLASS_NAME[] = "WinMain";
const char APP_TITLE[] = "Hello World"; //Title bar text
const int WINDOW_WIDTH = 400; //Width if the window
const int WINDOW_HEIGHT = 400; //Height of window

//=====================================================================
int WINAPI WinMain(HINSTANCE hInstance, 
				   HINSTANCE hPrevInstance, 
				   LPSTR     lpCmdLine, 
				   int       nCmdShow)
{
	MSG msg;

	//Create new window
	if (!CreateMainWindow(hInstance, nCmdShow))
		return false;

	//Main message loop
	int done = 0;
	while (!done)
	{
		if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
		{
			//Look for quit message
			if (msg.message == WM_QUIT)
				done = 1;
			//Decode and pass messages on to WinProc
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
	return msg.wParam;
}

//==============================================
//Window event callback function
//==============================================

LRESULT WINMAIN WinProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg)
	{
	case WM_DESTROY:
		//Tell windows to kill this program
		PostQuitMessage(0);
		return 0;
	}
	return DefWindowProc(hWnd, msg, wParam, lParam);
}

//=============================================
//Create the window
//Return: false on error
//=============================================

bool CreateMainWindow(HINSTANCE hInstance, int nCmdShow)
{
	WNDCLASSEX wcx;
	HWND hwnd;

	//Fill in the window class structure with parameters that describe the main window

	wcx.cbSize = sizeof(wcx);
	wcx.style = CS_HREDRAW | CS_VREDRAW;
	wcx.lpfnWndProc = WinProc;
	wcx.cbClsExtra = 0;
	wcx.cbWndExtra = 0;
	wcx.hInstance = hInstance;
	wcx.hIcon = NULL;
	wcx.hCursor = LoadCursor(NULL, IDC_ARROW);
	wcx.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
	wcx.lpszMenuName = NULL;
	wcx.lpszClassName = CLASS_NAME;
	wcx.hIconSm = NULL;

	//Register the window class
	//RegisterClassEx returns 0 on error
	if (RegisterClassEx(&wcx) == 0) //If error
	{
		return false;
	}
	
	//Create window
	hwnd = CreateWindow(CLASS_NAME,
						APP_TITLE,
						WS_OVERLAPPEDWINDOW,
						CW_USEDEFAULT,
						CW_USEDEFAULT,
						WINDOW_WIDTH,
						WINDOW_HEIGHT,
						(HWND)NULL,
						(HMENU)NULL,
						hInstance,
						(LPVOID)NULL);

	//If there was an error creating the window
	if (!hwnd)
	{
		return false;
	}	

	//Show the window
	ShowWindow(hwnd, nCmdShow);

	//Send a WM_PAINT message to the window procedure
	UpdateWindow(hwnd);
	
	return true;
}